import amazon
import re
import time

from AccessControl import ClassSecurityInfo, ModuleSecurityInfo
import DateTime
from ZODB.POSException import ConflictError
from Products.Archetypes.public import *
from Products.Archetypes.ExtensibleMetadata import ExtensibleMetadata
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.utils import getToolByName
from config import PROJECT_NAME, ADD_CONTENT_PERMISSION
from Widget import ReadOnlyFloatWidget, ReadOnlyIntegerWidget, ReadOnlyLinesWidget, ReadOnlyStringWidget

from AmazonData import AmazonData
from NullStorage import NullStorage
from utils import compareByRatingThenSalesRank

"""
      Asin - Amazon ID ("ASIN" number) of this item
      Authors - list of authors
      Availability - "available", etc.
      BrowseList - list of related categories
      Catalog - catalog type ("Book", etc)
      CollectiblePrice - ?, format "$34.95"
      ImageUrlLarge - URL of large image of this item
      ImageUrlMedium - URL of medium image of this item
      ImageUrlSmall - URL of small image of this item
      Isbn - ISBN number
      ListPrice - list price, format "$34.95"
      Lists - list of ListMania lists that include this item
      Manufacturer - manufacturer
      Media - media ("Paperback", "Audio CD", etc)
      NumMedia - number of different media types in which this item is available
      OurPrice - Amazon price, format "$24.47"
      ProductName - name of this item
      ReleaseDate - release date, format "09 April, 1999"
      Reviews - reviews (AvgCustomerRating, plus list of CustomerReview with Rating, Summary, Content)
      SalesRank - sales rank (integer)
      SimilarProducts - list of Product, which is ASIN number
      ThirdPartyNewPrice - ?, format "$34.95"
      URL - URL of this item
"""

content_schema = Schema((
    StringField('id',
                required=0, ## Still actually required, but
                            ## the widget will supply the missing value
                            ## on non-submits
                accessor='getId',
                mutator='setId',
                default=None,
                widget=IdWidget(label='Short Name',
                                label_msgid='label_short_name',
                                description='Short Name is part of the item\'s web address. '
                                            'Should not contain spaces, upper case, underscores '
                                            'or other special characters.',
                                description_msgid='help_shortname',
                                visible={'view' : 'invisible'},
                                i18n_domain="plone"),
               ),

    StringField('asin',
                accessor='getAsin',
                index = (':schema'),
                required=1,
                default='',
                widget=StringWidget(label='ASIN',
                                    description='Amazon ID (ASIN number) of this item',
                                    size=12,),
               ),

    StringField('title',
                accessor='Title',
                searchable=1,
                index = ('TextIndexNG|TextIndex:schema'),
                default='',
                storage=NullStorage(),
                widget=ReadOnlyStringWidget(label='Book Title',
                                            description=None,),
               ),

    TextField('main_page_description',
              default='',
              searchable=1,
              accessor='MainPageDescription',
              mutator='setMainPageDescription',
              storage=MetadataStorage(),
              widget = TextAreaWidget(description = 'Enter your notes for this item', 
                                      description_msgid = 'help_main_page_description',
                                      label = 'Notes',
                                      label_msgid = "label_description",
                                      rows = 6,
                                      cols = 60,
                                      i18n_domain = "plone"),
             ),

    LinesField('authors',
               accessor='getAuthors',
               searchable=1,
               default='',
               storage=NullStorage(),
               widget=ReadOnlyLinesWidget(label='Authors',
                                          description='List of authors',),
              ),

    FloatField('avg_customer_rating',
                accessor='getAvgCustomerRating',
                default='',
                storage=NullStorage(),
                widget=ReadOnlyFloatWidget(label='Average Customer Review',
                                           description=None,),
               ),

    StringField('availability',
                accessor='getAvailability',
                default='',
                storage=NullStorage(),
                widget=ReadOnlyStringWidget(label='Availability',
                                            description='"available", etc.',),
               ),

    LinesField('browse_list',
               accessor='getBrowseList',
               default='',
               storage=NullStorage(),
               widget=ReadOnlyLinesWidget(label='Browse List',
                                          description='A list of related categories',),
              ),

    StringField('catalog_type',
                accessor='getCatalogType',
                default='',
                storage=NullStorage(),
                widget=ReadOnlyStringWidget(label='Catalog Type',
                                            description='"Book", etc',),
               ),

    StringField('collectible_price',
                accessor='getCollectiblePrice',
                default='',
                storage=NullStorage(),
                widget=ReadOnlyStringWidget(label='Collectible Price',
                                            description='"Book", etc',),
               ),

    StringField('image_url_small',
                accessor='getImageUrlSmall',
                default='',
                storage=NullStorage(),
                widget=ReadOnlyStringWidget(label='Small Image URL',
                                            description='URL of small image of this item',),
               ),

    StringField('image_url_medium',
                accessor='getImageUrlMedium',
                default='',
                storage=NullStorage(),
                widget=ReadOnlyStringWidget(label='Medium Image URL',
                                            description='URL of medium image of this item',),
               ),

    StringField('image_url_large',
                accessor='getImageUrlLarge',
                default='',
                storage=NullStorage(),
                widget=ReadOnlyStringWidget(label='Large Image URL',
                                            description='URL of large image of this item',),
               ),

    StringField('isbn',
                accessor='getIsbn',
                default='',
                storage=NullStorage(),
                widget=ReadOnlyStringWidget(label='ISBN',
                                            description=None,),
               ),

    LinesField('lists',
               accessor='getLists',
               default='',
               storage=NullStorage(),
               widget=ReadOnlyLinesWidget(label='Lists',
                                          description='List of ListMania lists that include this item',),
               ),

    StringField('list_price',
                accessor='getListPrice',
                default='',
                storage=NullStorage(),
                widget=ReadOnlyStringWidget(label='List Price',
                                            description='List price, format "$34.95"',),
               ),

    StringField('manufacturer',
                accessor='getManufacturer',
                default='',
                storage=NullStorage(),
                widget=ReadOnlyStringWidget(label='Manufacturer',
                                            description=None,),
               ),

    StringField('media',
                accessor='getMedia',
                default='',
                storage=NullStorage(),
                widget=ReadOnlyStringWidget(label='Media',
                                            description='"Paperback", "Audio CD", etc',),
               ),

    IntegerField('num_media',
                 accessor='getNumMedia',
                 default=1,
                 storage=NullStorage(),
                 widget=ReadOnlyStringWidget(label='Number of Media',
                                            description='Number of different media types in which this item is available',),
               ),

    StringField('release_date',
                accessor='getReleaseDate',
                default='',
                storage=NullStorage(),
                widget=ReadOnlyStringWidget(label='Release Date',
                                            description='Release date, format "09 April, 1999"',),
               ),

    LinesField('similar_products',
               accessor='getSimilarProducts',
               default='',
               storage=NullStorage(),
               widget=ReadOnlyLinesWidget(label='Similar Products',
                                          description='List of ASIN numbers',),
              ),

    StringField('third_party_new_price',
                accessor='getThirdPartyNewPrice',
                default='',
                storage=NullStorage(),
                widget=ReadOnlyStringWidget(label='Third Party New Price',
                                            description=None,),
               ),

    StringField('third_party_used_price',
                accessor='getThirdPartyUsedPrice',
                default='',
                storage=NullStorage(),
                widget=ReadOnlyStringWidget(label='Third Party Used Price',
                                            description=None,),
               ),

    IntegerField('total_customer_reviews',
                 accessor='getTotalCustomerReviews',
                 default=0,
                 storage=NullStorage(),
                 widget=ReadOnlyIntegerWidget(label='Total Customer Reviews',
                                              description=None,),
               ),

    IntegerField('sales_rank',
                 accessor='getSalesRank',
                 index = (':schema'),
                 default=999999999,
                 storage=NullStorage(),
                 widget=ReadOnlyIntegerWidget(label='Sales Rank',
                                              description=None,),
               ),

    StringField('amazon_price',
                accessor='getAmazonPrice',
                default='',
                storage=NullStorage(),
                widget=ReadOnlyStringWidget(label='Amazon Price',
                                            description='Amazon price, format "$24.47"',),
               ),

    StringField('amazon_url',
                accessor='getAmazonUrl',
                default='',
                storage=NullStorage(),
                widget=ReadOnlyStringWidget(label='Amazon URL',
                                            description=None,),
               ),

    ))


# generate the addAmazonItem method ourselves so we can do some extra initialization
security = ModuleSecurityInfo('Products.ATAmazon.AmazonItem')
security.declareProtected(ADD_CONTENT_PERMISSION, 'addAmazonItem')
def addAmazonItem(self, id, **kwargs):
    o = AmazonItem(id)
    self._setObject(id, o)
    o = getattr(self, id)
    o.initializeArchetype(**kwargs)

    # if we have an ASIN in hand, set self.asin to id
    if self.isIDAutoGenerated(id): # id is autogenerated - not an ASIN
        return
    # try grabbing the object from Amazon
    at = getToolByName(self, 'amazon_tool')
    ad = at.cached_search_by_asin(id, True)
    if ad: # success - object is an ASIN
        o.asin = id

def _priceToFloat(p):
    p2 = ''
    for c in p:
        if (c >= '0' and c <= '9') or c == '.':
            p2 += c
    try:
        return float(p2)
    except (ValueError, TypeError):
        return 0.0

class AmazonItem(BaseContent):
    """ """
    security = ClassSecurityInfo()
    portal_type = meta_type = 'Amazon Item'
    schema = content_schema + ExtensibleMetadata.schema

    # initial values for all values in catalog metadata
    amazon_data = AmazonData()

    actions = ({ 'id': 'view',
                 'name': 'View',
                 'action': 'string:${object_url}/amazon_item_view',
                 'permissions': (CMFCorePermissions.View,),
                 'category': 'object',
               },
               { 'id': 'amazon_search',
                 'name': 'Search Amazon',
                 'action': 'string:${object_url}/search_amazon',
                 'permissions': (CMFCorePermissions.ModifyPortalContent,),
                 'category': 'object',
               },
               { 'id': 'amazon_similar',
                 'name': 'Similar Items',
                 'action': 'string:${object_url}/search_amazon_similar',
                 'permissions': (CMFCorePermissions.ModifyPortalContent,),
                 'category': 'object',
               },
               { 'id': 'details',
                 'name': 'Details',
                 'action': 'string:${object_url}/base_view',
                 'permissions': (CMFCorePermissions.ModifyPortalContent,),
                 'category': 'object',
               },
              )

    # immediate_view = 'search_amazon'
    def __init__(self, id):
           
        BaseContent.__init__(self, id)
        self.last_updated = 0

    def _update(self, heavy):
        """Make sure the current data object is up to date"""

        if not getattr(self, 'asin', None):
            return AmazonData()

        now = time.time()
        if heavy:
            attr = '_v_heavy'
            max_age = 24*3600
        else:
            attr = '_v_lite'
            max_age = 3600

        ob = getattr(self, attr, None)
        if ob and now - ob.last_updated < max_age:
            return ob
        # wait at least 60 seconds after a failed attempt before trying again
        last_failed_attempt = getattr(self, 'last_failed_attempt', None)
        if not last_failed_attempt or now - last_failed_attempt > 60:
            at = getToolByName(self, 'amazon_tool')
            new_ob = at.cached_search_by_asin(self.getAsin(), heavy)
            if new_ob:
                if hasattr(self, 'last_failed_attempt'):
                    delattr(self, 'last_failed_attempt')
                new_ob = new_ob[0]  # XXX really should grab the lowest price object
                setattr(self, attr, new_ob)
                if heavy:  # reindex when we do a fresh heavy search
                    if new_ob.last_updated > self.last_updated:
                        self.last_updated = new_ob.last_updated
                        # reindex if this is not a temporary object
                        if not getToolByName(self, 'portal_factory').isTemporary(self) and \
                           not self.getParentNode().getId() == 'amazon_tool' and \
                               self.getId() in self.getParentNode().objectIds():
                            self.reindexObject()
                return new_ob
            # could not get fresh object
            self.last_failed_attempt = now
        if ob:
            return ob
        return AmazonData()

    def Title(self):
        ad = self._update(heavy=True)
        return ad.getProductName(self.getCharset())

    def getAsin(self):
        asin = getattr(self, 'asin', None)
        if not asin:
            return asin
        return asin.strip()

    def getAuthors(self):
        ad = self._update(heavy=True)
        return ad.authors

    def getAvailability(self):
        ad = self._update(heavy=False)
        return ad.availability

    def getAvgCustomerRating(self):
        ad = self._update(heavy=True)
        if not type(ad.avg_customer_rating) == type(0.0):
            return 0.0
        return ad.avg_customer_rating

    def getBrowseList(self):
        ad = self._update(heavy=True)
        return ad.browse_list

    def getCatalogType(self):
        ad = self._update(heavy=True)
        return ad.catalog_type

    def getImageUrlSmall(self):
        ad = self._update(heavy=True)
        return ad.image_url_small

    def getImageUrlMedium(self):
        ad = self._update(heavy=True)
        return ad.image_url_medium

    def getImageUrlLarge(self):
        ad = self._update(heavy=True)
        return ad.image_url_large

    def getIsbn(self):
        ad = self._update(heavy=True)
        return ad.isbn

    def getListPrice(self):
        ad = self._update(heavy=False)
        return ad.list_price

    def getMedia(self):
        ad = self._update(heavy=True)
        return ad.media

    def getManufacturer(self):
        ad = self._update(heavy=True)
        return ad.manufacturer

    def getReleaseDate(self):
        ad = self._update(heavy=True)
        return ad.release_date

    def getReviews(self):
        ad = self._update(heavy=True)
        return ad.reviews

    def getSalesRank(self):
        ad = self._update(heavy=True)
        return ad.sales_rank

    def getSimilarProducts(self):
        ad = self._update(heavy=True)
        return ad.similar_products

    def getTotalCustomerReviews(self):
        ad = self._update(heavy=True)
        return ad.total_customer_reviews

    def getThirdPartyNewPrice(self):
        ad = self._update(heavy=False)
        return ad.third_party_new_price

    def getThirdPartyUsedPrice(self):
        ad = self._update(heavy=False)
        return ad.third_party_used_price

    def getAmazonPrice(self):
        ad = self._update(heavy=False)
        return ad.amazon_price

    def getAmazonUrl(self):
        ad = self._update(heavy=True)
        return ad.amazon_url

    def searchSimilarItems(self):
        ad = self._update(heavy=True)
        items = []
        parent = self.getParentNode()
        for id in ad.similar_products:
            item = AmazonItem(str(id)).__of__(parent)
            item.initializeArchetype()
            item.asin = id
            items.append(item)
        items.sort(compareByRatingThenSalesRank)
        return items

    def getAmazonSavings(self):
        lp = self.getListPrice()
        ap = self.getAmazonPrice()
        if lp and ap:
            currency = ap[0]
            saving = _priceToFloat(lp) - _priceToFloat(ap)
            if saving > 0:
                pcsaving = ((saving * 100.0) / _priceToFloat(lp))
                return "%s%.2f (%d%%)" % (currency, saving, pcsaving)
        return ''

    def _getThirdPartyLowPrice(self):
        up = self.getThirdPartyUsedPrice()
        np = self.getThirdPartyNewPrice()
        thirdParty = None
        currency = None
        if up:
            used3 = _priceToFloat(up)
            currency = up[0]
            thirdParty = used3
        if np:
            new3 = _priceToFloat(np)
            if thirdParty is None or new3 < thirdParty:
                currency = np[0]
                thirdParty = new3
        return (currency, thirdParty)
        
    def getThirdPartyLowPrice(self):
        (currency, thirdParty) = self._getThirdPartyLowPrice()
        if thirdParty is None:
            return ''
        return "%s%.2f" %(currency, thirdParty)


    def getThirdPartySavings(self):
        lp = self.getListPrice()
        listPrice = None
        if lp:
            listPrice = _priceToFloat(lp)
            
        (currency, thirdParty) = self._getThirdPartyLowPrice()
        if listPrice and thirdParty and thirdParty < listPrice:
            listPrice = _priceToFloat(lp)
            saving = listPrice - thirdParty
            if saving > 0:
                pcsaving = ((saving * 100.0) / listPrice)
                return "%s%.2f (%d%%)" % (currency, saving, pcsaving)
        return ''

registerType(AmazonItem, PROJECT_NAME)
