import time
from Persistence import Persistent

class SimpleCache(Persistent):

    def __init__(self, max_age, max_size, cleanup_interval):
        self.max_age = max_age
        self.max_size = max_size
        self.cleanup_interval = cleanup_interval

        self._data = {}
        self.last_cleanup = time.time()
    
    def get(self, key, max_age=None):
        if max_age is None:
            max_age = self.max_age
            
        self.cleanup()

        (t, v) = self._data.get(key, (None, None))

        if t:
            now = time.time()
            if now - t < max_age:
                return v
            del self._data[key]
        return None

    def set(self, key, value):
        self.cleanup()

        now = time.time()
    
        # brute force cache cleanup if things get too big 
        if len(self._data) >= self.max_size:
            self._data.clear()

        self._data[key] = (now, value)
        self._p_changed = 1

    def cleanup(self):
        to_be_deleted = []
        now = time.time()
        if now - self.last_cleanup > self.cleanup_interval:
            for (k, (t, v)) in self._data.items():
                if now - t > self.max_age:
                    to_be_deleted.append(k)
        for k in to_be_deleted:
            del self._data[k]

        # still too big after cleanup?  nuke everything!
        if len(self._data) >= self.max_size:
            self._data.clear()
            
        self.last_cleanup = now
        self._p_changed = 1
