## Script (Python) "getAmazonItems"
##title=Get a list of AmazonItems to display
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=n=None, item_folder_id=None

# This script is fairly expensive to call, so make sure the results are
# cached.  If you display pricing information in the portlet, you should
# cache for no more than 1 hour to comply with Amazon's licensing terms.
# Other fields may be cached for longer periods of time -- see the Amazon
# web services license agreement for details.


from Products.ATAmazon.utils import compareByRatingThenSalesRank

def getParentItems(parent, items, asins, n):
    parent_items = parent.getAmazonItems(n)
    new_items = []
    for c in parent_items:
        asin = c.getAsin()
        if not asin in asins:
            new_items.append(c)
            asins[asin] = 1
            if len(items) + len(new_items) == n:
                break
    new_items.sort(compareByRatingThenSalesRank)
    items += new_items


def getItemsFromFolder(folder, items, asins, n):
    new_items = []
    for c in folder.contentValues(spec='Amazon Item'):
        asin = c.getAsin()
        if not asins.has_key(asin):
            new_items.append(c)
            asins[asin] = 1
            if len(items) + len(new_items) == n:
                break
    new_items.sort(compareByRatingThenSalesRank)
    items += new_items

def getItemsFromCatalog(path, items, asins, n):
    # Grab amazon items from the catalog, sort them by sales rank, then add
    # any new items to our list of items.  Instantiate the objects as we add
    # them to our list so their info can get updated from Amazon if need be.
    results = context.portal_catalog.searchResults(portal_type='Amazon Item', path=path, sort_index='getSalesRank')
    new_items = []
    for r in results:
        asin = r.getAsin
        if not asins.has_key(asin):
            o = r.getObject()
            if o is not None:  # allow for corrupted catalog entries
                new_items.append(r.getObject())
                asins[asin] = 1
                if len(items) + len(new_items) == n:
                    break
    new_items.sort(compareByRatingThenSalesRank)
    items += new_items

amazon_tool = context.amazon_tool
if n is None:
    n = amazon_tool.n_items_in_portlet
if item_folder_id is None:
    item_folder_id = amazon_tool.amazon_item_folder_id

folder = context
pf = context.portal_factory
while (pf.isTemporary(folder) or not folder.isPrincipiaFolderish):
    folder = folder.getParentNode()
if folder.getId() == item_folder_id:
    folder = folder.getParentNode()

items = []
asins = {}
if n == 0:
    return items
# check for a folder with id item_folder_id
if item_folder_id in folder.objectIds():
    getItemsFromFolder(getattr(folder, item_folder_id), items, asins, n)

if len(items) < n:
    # not enough items - look in the current folder
    getItemsFromFolder(folder, items, asins, n)

    if len(items) < n:
        # not enough items - look in folders below this one
        urltool = context.portal_url
        getItemsFromCatalog('/'.join(folder.getPhysicalPath()), items, asins, n)

        if len(items) < n:
            # still not enough - go to parent
            portal = context.portal_url.getPortalObject()
            if folder != portal:
                getParentItems(folder.getParentNode(), items, asins, n)

return items
